@@ -59,6 +59,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
 import static org.apache.calcite.rel.RelCollations.containsOrderless;
 
 import static java.util.Objects.requireNonNull;
@@ -94,7 +96,8 @@ public class EnumerableMergeJoin extends Join implements EnumerableRel {
 
     final List<RelCollation> collations =
         traits.getTraits(RelCollationTraitDef.INSTANCE);
-    assert collations != null && collations.size() > 0;
+    requireNonNull(collations, "collations");
+    checkArgument(!collations.isEmpty());
     ImmutableIntList rightKeys = joinInfo.rightKeys
         .incr(left.getRowType().getFieldCount());
     // Currently it has very limited ability to represent the equivalent traits
@@ -347,11 +350,9 @@ public class EnumerableMergeJoin extends Join implements EnumerableRel {
       keyMap.put(sourceKeys.get(i), targetKeys.get(i));
     }
 
-    Mappings.TargetMapping mapping =
-        Mappings.target(keyMap,
-            (left2Right ? left : right).getRowType().getFieldCount(),
-            (left2Right ? right : left).getRowType().getFieldCount());
-    return mapping;
+    return Mappings.target(keyMap,
+        (left2Right ? left : right).getRowType().getFieldCount(),
+        (left2Right ? right : left).getRowType().getFieldCount());
   }
 
   /**
@@ -424,8 +425,8 @@ public class EnumerableMergeJoin extends Join implements EnumerableRel {
     // We assume that the inputs are sorted. The price of sorting them has
     // already been paid. The cost of the join is therefore proportional to the
     // input and output size.
-    final double rightRowCount = right.estimateRowCount(mq);
-    final double leftRowCount = left.estimateRowCount(mq);
+    final double rightRowCount = mq.getRowCount(right);
+    final double leftRowCount = mq.getRowCount(left);
     final double rowCount = mq.getRowCount(this);
     final double d = leftRowCount + rightRowCount + rowCount;
     return planner.getCostFactory().makeCost(d, 0, 0);
