@@ -22,22 +22,27 @@ import org.apache.calcite.linq4j.tree.Expressions;
 import org.apache.calcite.schema.Schema;
 import org.apache.calcite.schema.SchemaPlus;
 import org.apache.calcite.schema.Schemas;
+import org.apache.calcite.schema.Table;
 import org.apache.calcite.schema.Wrapper;
-import org.apache.calcite.schema.impl.AbstractSchema;
+import org.apache.calcite.schema.lookup.IgnoreCaseLookup;
+import org.apache.calcite.schema.lookup.LikePattern;
+import org.apache.calcite.schema.lookup.LoadingCacheLookup;
+import org.apache.calcite.schema.lookup.Lookup;
 import org.apache.calcite.sql.SqlDialect;
 import org.apache.calcite.sql.SqlDialectFactory;
 import org.apache.calcite.sql.SqlDialectFactoryImpl;
 import org.apache.calcite.util.BuiltInMethod;
 
 import com.google.common.base.Suppliers;
-import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.ImmutableSet;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
 
 import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.SQLException;
-import java.util.Map;
+import java.util.Optional;
+import java.util.Set;
 import java.util.function.Supplier;
 import javax.sql.DataSource;
 
@@ -51,20 +56,21 @@ import static java.util.Objects.requireNonNull;
  * an instance of {@link JdbcSchema}.
  *
  * <p>This schema is lazy: it does not compute the list of schema names until
- * the first call to {@link #getSubSchemaMap()}. Then it creates a
- * {@link JdbcSchema} for each schema name. Each JdbcSchema will populate its
+ * the first call to {@link #subSchemas()} and {@link Lookup#get(String)}. Then it creates a
+ * {@link JdbcSchema} for this schema name. Each JdbcSchema will populate its
  * tables on demand.
  */
-public class JdbcCatalogSchema extends AbstractSchema implements Wrapper {
+public class JdbcCatalogSchema extends JdbcBaseSchema implements Wrapper {
   final DataSource dataSource;
   public final SqlDialect dialect;
   final JdbcConvention convention;
   final String catalog;
+  private final Lookup<JdbcSchema> subSchemas;
 
-  /** Sub-schemas by name, lazily initialized. */
+  /** default schema name, lazily initialized. */
   @SuppressWarnings({"method.invocation.invalid", "Convert2MethodRef"})
-  final Supplier<SubSchemaMap> subSchemaMapSupplier =
-      Suppliers.memoize(() -> computeSubSchemaMap());
+  private final Supplier<Optional<String>> defaultSchemaName =
+      Suppliers.memoize(() -> Optional.ofNullable(computeDefaultSchemaName()));
 
   /** Creates a JdbcCatalogSchema. */
   public JdbcCatalogSchema(DataSource dataSource, SqlDialect dialect,
@@ -73,6 +79,40 @@ public class JdbcCatalogSchema extends AbstractSchema implements Wrapper {
     this.dialect = requireNonNull(dialect, "dialect");
     this.convention = requireNonNull(convention, "convention");
     this.catalog = catalog;
+    this.subSchemas = new LoadingCacheLookup<>(new IgnoreCaseLookup<JdbcSchema>() {
+      @Override public @Nullable JdbcSchema get(String name) {
+        try (Connection connection = dataSource.getConnection();
+            ResultSet resultSet =
+                connection.getMetaData().getSchemas(catalog, name)) {
+          while (resultSet.next()) {
+            final String schemaName =
+                requireNonNull(resultSet.getString(1),
+                    "got null schemaName from the database");
+            return new JdbcSchema(dataSource, dialect, convention, catalog, schemaName);
+          }
+        } catch (SQLException e) {
+          throw new RuntimeException(e);
+        }
+        return null;
+      }
+
+      @Override public Set<String> getNames(LikePattern pattern) {
+        final ImmutableSet.Builder<String> builder =
+            ImmutableSet.builder();
+        try (Connection connection = dataSource.getConnection();
+            ResultSet resultSet =
+                connection.getMetaData().getSchemas(catalog, pattern.pattern)) {
+          while (resultSet.next()) {
+            builder.add(
+                requireNonNull(resultSet.getString(1),
+                    "got null schemaName from the database"));
+          }
+        } catch (SQLException e) {
+          throw new RuntimeException(e);
+        }
+        return builder.build();
+      }
+    });
   }
 
   public static JdbcCatalogSchema create(
@@ -103,34 +143,25 @@ public class JdbcCatalogSchema extends AbstractSchema implements Wrapper {
     return new JdbcCatalogSchema(dataSource, dialect, convention, catalog);
   }
 
-  private SubSchemaMap computeSubSchemaMap() {
-    final ImmutableMap.Builder<String, Schema> builder =
-        ImmutableMap.builder();
-    @Nullable String defaultSchemaName;
-    try (Connection connection = dataSource.getConnection();
-         ResultSet resultSet =
-             connection.getMetaData().getSchemas(catalog, null)) {
-      defaultSchemaName = connection.getSchema();
-      while (resultSet.next()) {
-        final String schemaName =
-            requireNonNull(resultSet.getString(1),
-                "got null schemaName from the database");
-        builder.put(schemaName,
-            new JdbcSchema(dataSource, dialect, convention, catalog, schemaName));
-      }
+  @Override public Lookup<Table> tables() {
+    return Lookup.empty();
+  }
+
+  @Override public Lookup<? extends Schema> subSchemas() {
+    return subSchemas;
+  }
+
+  private @Nullable String computeDefaultSchemaName() {
+    try (Connection connection = dataSource.getConnection()) {
+      return connection.getSchema();
     } catch (SQLException e) {
       throw new RuntimeException(e);
     }
-    return new SubSchemaMap(defaultSchemaName, builder.build());
-  }
-
-  @Override protected Map<String, Schema> getSubSchemaMap() {
-    return subSchemaMapSupplier.get().map;
   }
 
   /** Returns the name of the default sub-schema. */
   public @Nullable String getDefaultSubSchemaName() {
-    return subSchemaMapSupplier.get().defaultSchemaName;
+    return defaultSchemaName.get().orElse(null);
   }
 
   /** Returns the data source. */
@@ -148,16 +179,4 @@ public class JdbcCatalogSchema extends AbstractSchema implements Wrapper {
     }
     return null;
   }
-
-  /** Contains sub-schemas by name, and the name of the default schema. */
-  private static class SubSchemaMap {
-    final @Nullable String defaultSchemaName;
-    final ImmutableMap<String, Schema> map;
-
-    private SubSchemaMap(@Nullable String defaultSchemaName,
-        ImmutableMap<String, Schema> map) {
-      this.defaultSchemaName = defaultSchemaName;
-      this.map = map;
-    }
-  }
 }
