@@ -21,6 +21,7 @@ import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.metadata.RelMetadataProvider;
 import org.apache.calcite.rel.metadata.RelMetadataQuery;
 import org.apache.calcite.rex.RexExecutor;
+import org.apache.calcite.sql2rel.RelDecorrelator;
 import org.apache.calcite.util.CancelFlag;
 import org.apache.calcite.util.Pair;
 import org.apache.calcite.util.Util;
@@ -86,6 +87,8 @@ public abstract class AbstractRelOptPlanner implements RelOptPlanner {
 
   private @Nullable RexExecutor executor;
 
+  private @Nullable RelDecorrelator decorrelator;
+
   //~ Constructors -----------------------------------------------------------
 
   /**
@@ -290,6 +293,17 @@ public abstract class AbstractRelOptPlanner implements RelOptPlanner {
     return executor;
   }
 
+  @Override public void setDecorrelator(@Nullable RelDecorrelator decorrelator) {
+    this.decorrelator = decorrelator;
+  }
+
+  @Override public RelDecorrelator getDecorrelator() {
+    if (decorrelator == null) {
+      throw new IllegalStateException("RelDecorrelator has not been set");
+    }
+    return decorrelator;
+  }
+
   @Override public void onCopy(RelNode rel, RelNode newRel) {
     // do nothing
   }
