@@ -38,6 +38,8 @@ import java.util.List;
 
 import static com.google.common.base.Preconditions.checkArgument;
 
+import static org.apache.calcite.sql.SqlKind.SET_QUERY;
+
 /**
  * <code>SetOp</code> is an abstract base for relational set operators such
  * as UNION, MINUS (aka EXCEPT), and INTERSECT.
@@ -58,9 +60,7 @@ public abstract class SetOp extends AbstractRelNode implements Hintable {
   protected SetOp(RelOptCluster cluster, RelTraitSet traits, List<RelHint> hints,
       List<RelNode> inputs, SqlKind kind, boolean all) {
     super(cluster, traits);
-    checkArgument(kind == SqlKind.UNION
-        || kind == SqlKind.INTERSECT
-        || kind == SqlKind.EXCEPT);
+    checkArgument(SET_QUERY.contains(kind));
     this.kind = kind;
     this.inputs = ImmutableList.copyOf(inputs);
     this.all = all;
