@@ -98,6 +98,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.UUID;
 
 import static org.apache.calcite.rel.RelDistributions.EMPTY;
 import static org.apache.calcite.util.Static.RESOURCE;
@@ -493,8 +494,12 @@ public class RelJson {
     } else if (value instanceof Range) {
       //noinspection rawtypes,unchecked
       return toJson((Range) value);
+    } else if (value instanceof ByteString) {
+      return toJson(((ByteString) value).toString(16));
+    } else if (value instanceof UUID) {
+      return toJson(value.toString());
     } else {
-      throw new UnsupportedOperationException("type not serializable: "
+      throw new UnsupportedOperationException("type not serializable as JSON: "
           + value + " (type " + value.getClass().getCanonicalName() + ")");
     }
   }
@@ -825,12 +830,15 @@ public class RelJson {
         SqlTypeName sqlTypeName = type.getSqlTypeName();
         if (sqlTypeName == SqlTypeName.SYMBOL) {
           literal = RelEnumTypes.toEnum((String) literal);
-        }
-        if (sqlTypeName == SqlTypeName.TIMESTAMP
+        } else if (sqlTypeName == SqlTypeName.TIMESTAMP
             || sqlTypeName == SqlTypeName.TIMESTAMP_WITH_LOCAL_TIME_ZONE) {
           if (literal instanceof Integer) {
             literal = ((Integer) literal).longValue();
           }
+        } else if (sqlTypeName == SqlTypeName.BINARY || sqlTypeName == SqlTypeName.VARBINARY) {
+          literal = ByteString.of((String) literal, 16);
+        } else if (sqlTypeName == SqlTypeName.UUID) {
+          literal = UUID.fromString((String) literal);
         }
         return rexBuilder.makeLiteral(literal, type);
       }
