@@ -38,6 +38,8 @@ import org.apache.calcite.rex.RexCall;
 import org.apache.calcite.rex.RexInputRef;
 import org.apache.calcite.rex.RexLiteral;
 import org.apache.calcite.rex.RexNode;
+import org.apache.calcite.sql.SqlKind;
+import org.apache.calcite.sql.type.SqlTypeUtil;
 import org.apache.calcite.util.ImmutableNullableList;
 import org.apache.calcite.util.NlsString;
 import org.apache.calcite.util.Pair;
@@ -49,6 +51,8 @@ import org.checkerframework.checker.nullness.qual.Nullable;
 import java.util.ArrayList;
 import java.util.List;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Default implementations of the
  * {@link org.apache.calcite.rel.metadata.BuiltInMetadata.Size}
@@ -410,6 +414,16 @@ public class RelMdSize implements MetadataHandler<BuiltInMetadata.Size> {
     default:
       if (node instanceof RexCall) {
         RexCall call = (RexCall) node;
+
+        // ARRAY constructor: array size multiplied by the average size of its component type
+        if (call.isA(SqlKind.ARRAY_VALUE_CONSTRUCTOR)) {
+          assert SqlTypeUtil.isArray(call.getType());
+          Double compSize = averageTypeValueSize(requireNonNull(call.getType().getComponentType()));
+          if (compSize != null) {
+            return compSize * call.operands.size();
+          }
+        }
+
         for (RexNode operand : call.getOperands()) {
           // It's a reasonable assumption that a function's result will have
           // similar size to its argument of a similar type. For example,
