@@ -46,7 +46,7 @@ import org.apache.calcite.util.Util;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMultimap;
 import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Maps;
+import com.google.common.collect.Multimap;
 import com.google.common.collect.Sets;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
@@ -54,7 +54,6 @@ import org.checkerframework.checker.nullness.qual.Nullable;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Map;
 import java.util.Set;
 import java.util.stream.Collectors;
 
@@ -63,21 +62,44 @@ import static com.google.common.base.Preconditions.checkArgument;
 import static org.apache.calcite.rel.metadata.RelMdColumnUniqueness.PASSTHROUGH_AGGREGATIONS;
 import static org.apache.calcite.rel.metadata.RelMdColumnUniqueness.getConstantColumnSet;
 
-import static java.util.Objects.requireNonNull;
-
 /**
  * RelMdUniqueKeys supplies a default implementation of
  * {@link RelMetadataQuery#getUniqueKeys} for the standard logical algebra.
+ * The number of returned keys for each relational expression is bounded by a limit.
+ * The limit is used to restrict the exponential logic that can appear for certain query patterns
+ * and lead to CPU/memory exhaustion and crashes.
  */
 public class RelMdUniqueKeys
     implements MetadataHandler<BuiltInMetadata.UniqueKeys> {
   public static final RelMetadataProvider SOURCE =
       ReflectiveRelMetadataProvider.reflectiveSource(
           new RelMdUniqueKeys(), BuiltInMetadata.UniqueKeys.Handler.class);
-
+  /**
+   * A limit about the number of unique keys returned by the handler.
+   * The limit must be in the range [0, Integer.MAX_VALUE].
+   */
+  private final int limit;
   //~ Constructors -----------------------------------------------------------
 
-  private RelMdUniqueKeys() {}
+  /**
+   * Creates a metadata handler for unique keys with the default limit.
+   */
+  public RelMdUniqueKeys() {
+    this(1000);
+  }
+
+  /**
+   * Creates a metadata handler for unique keys with the specified limit.
+   *
+   * @param limit a non-negative integer that bounds the number of unique keys returned for each
+   *              relational expression.
+   */
+  public RelMdUniqueKeys(int limit) {
+    if (limit < 0) {
+      throw new IllegalArgumentException("Limit cannot be negative");
+    }
+    this.limit = limit;
+  }
 
   //~ Methods ----------------------------------------------------------------
 
@@ -105,6 +127,9 @@ public class RelMdUniqueKeys
 
   public @Nullable Set<ImmutableBitSet> getUniqueKeys(Sort rel, RelMetadataQuery mq,
       boolean ignoreNulls) {
+    if (limit == 0) {
+      return ImmutableSet.of();
+    }
     Double maxRowCount = mq.getMaxRowCount(rel);
     if (maxRowCount != null && maxRowCount <= 1.0d) {
       return ImmutableSet.of(ImmutableBitSet.of());
@@ -134,7 +159,7 @@ public class RelMdUniqueKeys
         Util.transform(program.getProjectList(), program::expandLocalRef));
   }
 
-  private static Set<ImmutableBitSet> getProjectUniqueKeys(SingleRel rel, RelMetadataQuery mq,
+  private Set<ImmutableBitSet> getProjectUniqueKeys(SingleRel rel, RelMetadataQuery mq,
       boolean ignoreNulls, List<RexNode> projExprs) {
     // LogicalProject maps a set of rows to a different set;
     // Without knowledge of the mapping function(whether it
@@ -172,33 +197,30 @@ public class RelMdUniqueKeys
       return ImmutableSet.of();
     }
 
-    Map<Integer, ImmutableBitSet> mapInToOutPos =
-        Maps.transformValues(inToOutPosBuilder.build().asMap(), ImmutableBitSet::of);
+    Multimap<Integer, Integer> mapInToOutPos = inToOutPosBuilder.build();
 
-    ImmutableSet.Builder<ImmutableBitSet> resultBuilder = ImmutableSet.builder();
+    Set<ImmutableBitSet> resultBuilder = new HashSet<>();
     // Now add to the projUniqueKeySet the child keys that are fully
     // projected.
+    outerLoop:
     for (ImmutableBitSet colMask : childUniqueKeySet) {
       if (!inColumnsUsed.contains(colMask)) {
         // colMask contains a column that is not projected as RexInput => the key is not unique
         continue;
       }
       // colMask is mapped to output project, however, the column can be mapped more than once:
-      // select id, id, id, unique2, unique2
-      // the resulting unique keys would be {{0},{3}}, {{0},{4}}, {{0},{1},{4}}, ...
+      // select key1, key1, val1, val2, key2 from ...
+      // the resulting unique keys would be {{0},{4}}, {{1},{4}}
 
-      Iterable<List<ImmutableBitSet>> product =
-          Linq4j.product(
-              Util.transform(colMask, in ->
-                  Util.filter(
-                      requireNonNull(mapInToOutPos.get(in),
-                          () -> "no entry for column " + in
-                              + " in mapInToOutPos: " + mapInToOutPos).powerSet(),
-                      bs -> !bs.isEmpty())));
-
-      resultBuilder.addAll(Util.transform(product, ImmutableBitSet::union));
+      Iterable<List<Integer>> product = Linq4j.product(Util.transform(colMask, mapInToOutPos::get));
+      for (List<Integer> passKey : product) {
+        if (resultBuilder.size() == limit) {
+          break outerLoop;
+        }
+        resultBuilder.add(ImmutableBitSet.of(passKey));
+      }
     }
-    return resultBuilder.build();
+    return resultBuilder;
   }
 
   public @Nullable Set<ImmutableBitSet> getUniqueKeys(Join rel, RelMetadataQuery mq,
@@ -283,17 +305,7 @@ public class RelMdUniqueKeys
           .forEach(retSet::add);
     }
 
-    // Remove sets that are supersets of other sets
-    final Set<ImmutableBitSet> reducedSet = new HashSet<>();
-    for (ImmutableBitSet bigger : retSet) {
-      if (retSet.stream()
-          .filter(smaller -> !bigger.equals(smaller))
-          .noneMatch(bigger::contains)) {
-        reducedSet.add(bigger);
-      }
-    }
-
-    return reducedSet;
+    return filterSupersets(retSet, limit);
   }
 
   /**
@@ -344,15 +356,23 @@ public class RelMdUniqueKeys
 
       // If an input's unique column(s) value is returned (passed through) by an aggregation
       // function, then the result of the function(s) is also unique.
-      final ImmutableSet.Builder<ImmutableBitSet> keysBuilder = ImmutableSet.builder();
+      Set<ImmutableBitSet> keysBuilder = new HashSet<>();
       if (inputUniqueKeys != null) {
+        outerLoop:
         for (ImmutableBitSet inputKey : inputUniqueKeys) {
-          keysBuilder.addAll(getPassedThroughCols(inputKey, rel));
+          Iterable<List<Integer>> product =
+              Linq4j.product(Util.transform(inputKey, i -> getPassedThroughCols(i, rel)));
+          for (List<Integer> passKey : product) {
+            if (keysBuilder.size() == limit) {
+              break outerLoop;
+            }
+            keysBuilder.add(ImmutableBitSet.of(passKey));
+          }
         }
       }
 
-      return filterSupersets(Sets.union(preciseUniqueKeys, keysBuilder.build()));
-    } else if (ignoreNulls) {
+      return filterSupersets(Sets.union(preciseUniqueKeys, keysBuilder), limit);
+    } else if (ignoreNulls && limit > 0) {
       // group by keys form a unique key
       return ImmutableSet.of(rel.getGroupSet());
     } else {
@@ -367,7 +387,7 @@ public class RelMdUniqueKeys
    * other keys.  Given {@code {0},{1},{1,2}}, returns {@code {0},{1}}.
    */
   private static Set<ImmutableBitSet> filterSupersets(
-      Set<ImmutableBitSet> uniqueKeys) {
+      Set<ImmutableBitSet> uniqueKeys, int limit) {
     Set<ImmutableBitSet> minimalKeys = new HashSet<>();
     outer:
     for (ImmutableBitSet candidateKey : uniqueKeys) {
@@ -377,35 +397,14 @@ public class RelMdUniqueKeys
           continue outer;
         }
       }
+      if (minimalKeys.size() == limit) {
+        break outer;
+      }
       minimalKeys.add(candidateKey);
     }
     return minimalKeys;
   }
 
-  /**
-   * Given a set of columns in the input of an Aggregate rel, returns the set of mappings from the
-   * input columns to the output of the aggregations. A mapping for a particular column exists if
-   * it is part of a simple group by and/or it is "passed through" unmodified by a
-   * {@link RelMdColumnUniqueness#PASSTHROUGH_AGGREGATIONS pass-through aggregation function}.
-   */
-  private static Set<ImmutableBitSet> getPassedThroughCols(
-      ImmutableBitSet inputColumns, Aggregate rel) {
-    checkArgument(Aggregate.isSimple(rel));
-    Set<ImmutableBitSet> conbinations = new HashSet<>();
-    conbinations.add(ImmutableBitSet.of());
-    for (Integer inputColumn : inputColumns.asSet()) {
-      final ImmutableBitSet passedThroughCols = getPassedThroughCols(inputColumn, rel);
-      final Set<ImmutableBitSet> crossProduct = new HashSet<>();
-      for (ImmutableBitSet set : conbinations) {
-        for (Integer passedThroughCol : passedThroughCols) {
-          crossProduct.add(set.rebuild().set(passedThroughCol).build());
-        }
-      }
-      conbinations = crossProduct;
-    }
-    return conbinations;
-  }
-
   /**
    * Given a column in the input of an Aggregate rel, returns the mappings from the input column to
    * the output of the aggregations. A mapping for the column exists if it is part of a simple
@@ -414,6 +413,7 @@ public class RelMdUniqueKeys
    */
   private static ImmutableBitSet getPassedThroughCols(Integer inputColumn,
       Aggregate rel) {
+    checkArgument(Aggregate.isSimple(rel));
     final ImmutableBitSet.Builder builder = ImmutableBitSet.builder();
     if (rel.getGroupSet().get(inputColumn)) {
       builder.set(inputColumn);
@@ -431,7 +431,7 @@ public class RelMdUniqueKeys
 
   public Set<ImmutableBitSet> getUniqueKeys(Union rel, RelMetadataQuery mq,
       boolean ignoreNulls) {
-    if (!rel.all) {
+    if (!rel.all && limit > 0) {
       return ImmutableSet.of(
           ImmutableBitSet.range(rel.getRowType().getFieldCount()));
     }
@@ -443,19 +443,24 @@ public class RelMdUniqueKeys
    */
   public Set<ImmutableBitSet> getUniqueKeys(Intersect rel,
       RelMetadataQuery mq, boolean ignoreNulls) {
-    ImmutableSet.Builder<ImmutableBitSet> keys = new ImmutableSet.Builder<>();
+    Set<ImmutableBitSet> keys = new HashSet<>();
+    outerLoop:
     for (RelNode input : rel.getInputs()) {
       Set<ImmutableBitSet> uniqueKeys = mq.getUniqueKeys(input, ignoreNulls);
       if (uniqueKeys != null) {
-        keys.addAll(uniqueKeys);
+        for (ImmutableBitSet inKey : uniqueKeys) {
+          if (keys.size() == limit) {
+            break outerLoop;
+          }
+          keys.add(inKey);
+        }
       }
     }
-    ImmutableSet<ImmutableBitSet> uniqueKeys = keys.build();
-    if (!uniqueKeys.isEmpty()) {
-      return uniqueKeys;
+    if (!keys.isEmpty()) {
+      return keys;
     }
 
-    if (!rel.all) {
+    if (!rel.all && limit > 0) {
       return ImmutableSet.of(
           ImmutableBitSet.range(rel.getRowType().getFieldCount()));
     }
@@ -472,7 +477,7 @@ public class RelMdUniqueKeys
       return uniqueKeys;
     }
 
-    if (!rel.all) {
+    if (!rel.all && limit > 0) {
       return ImmutableSet.of(
           ImmutableBitSet.range(rel.getRowType().getFieldCount()));
     }
@@ -491,10 +496,15 @@ public class RelMdUniqueKeys
     if (keys == null) {
       return null;
     }
+    Set<ImmutableBitSet> result = new HashSet<>(Math.min(keys.size(), limit));
     for (ImmutableBitSet key : keys) {
+      if (result.size() == limit) {
+        break;
+      }
       assert rel.getTable().isKey(key);
+      result.add(key);
     }
-    return ImmutableSet.copyOf(keys);
+    return result;
   }
 
   public @Nullable Set<ImmutableBitSet> getUniqueKeys(Values rel, RelMetadataQuery mq,
@@ -516,10 +526,15 @@ public class RelMdUniqueKeys
     }
 
     ImmutableSet.Builder<ImmutableBitSet> keySetBuilder = ImmutableSet.builder();
+    int keySetSize = 0;
     for (int i = 0; i < ranges.size(); i++) {
       final Set<RexLiteral> range = ranges.get(i);
+      if (keySetSize == limit) {
+        break;
+      }
       if (range.size() == tuples.size()) {
         keySetBuilder.add(ImmutableBitSet.of(i));
+        keySetSize++;
       }
     }
     return keySetBuilder.build();
