@@ -320,7 +320,10 @@ public class AggregateJoinTransposeRule
     // Update condition
     final Mapping mapping =
         (Mapping) Mappings.target(map::get,
-            join.getRowType().getFieldCount(),
+            join.getJoinType().projectsRight()
+                ? join.getRowType().getFieldCount()
+                : join.getLeft().getRowType().getFieldCount()
+                    + join.getRight().getRowType().getFieldCount(),
             belowOffset);
     final RexNode newCondition =
         RexUtil.apply(mapping, join.getCondition());
