@@ -51,10 +51,8 @@ public class RexProgramBuilder {
   private final List<RexLocalRef> localRefList = new ArrayList<>();
   private final List<RexLocalRef> projectRefList = new ArrayList<>();
   private final List<@Nullable String> projectNameList = new ArrayList<>();
-  @SuppressWarnings("unused")
-  private final @Nullable RexSimplify simplify;
   private @Nullable RexLocalRef conditionRef = null;
-  private boolean validating;
+  private final boolean validating;
 
   //~ Constructors -----------------------------------------------------------
 
@@ -70,17 +68,16 @@ public class RexProgramBuilder {
    */
   @SuppressWarnings("method.invocation.invalid")
   private RexProgramBuilder(RelDataType inputRowType, RexBuilder rexBuilder,
-      @Nullable RexSimplify simplify) {
+      @Nullable RexSimplify unusedSimplify) {
     this.inputRowType = requireNonNull(inputRowType, "inputRowType");
     this.rexBuilder = requireNonNull(rexBuilder, "rexBuilder");
-    this.simplify = simplify; // may be null
     this.validating = assertionsAreEnabled();
 
     // Pre-create an expression for each input field.
     if (inputRowType.isStruct()) {
       final List<RelDataTypeField> fields = inputRowType.getFieldList();
       for (int i = 0; i < fields.size(); i++) {
-        registerInternal(RexInputRef.of(i, fields), false);
+        registerInternal(RexInputRef.of(i, fields));
       }
     }
   }
@@ -272,7 +269,7 @@ public class RexProgramBuilder {
    * not, call {@link #registerOutput(RexNode)} first.
    */
   public void addCondition(RexNode expr) {
-    assert expr != null;
+    requireNonNull(expr, "expr");
     RexLocalRef conditionRef = this.conditionRef;
     if (conditionRef == null) {
       this.conditionRef = conditionRef = registerInput(expr);
@@ -322,14 +319,9 @@ public class RexProgramBuilder {
    * Registers an expression in the list of common sub-expressions, and
    * returns a reference to that expression.
    *
-   * <p>If an equivalent sub-expression already exists, creates another
-   * expression only if <code>force</code> is true.
-   *
-   * @param expr  Expression to register
-   * @param force Whether to create a new sub-expression if an equivalent
-   *              sub-expression exists.
+   * @param expr Expression to register
    */
-  private RexLocalRef registerInternal(RexNode expr, boolean force) {
+  private RexLocalRef registerInternal(RexNode expr) {
     final RexSimplify simplify =
         new RexSimplify(rexBuilder, RelOptPredicateList.EMPTY, RexUtil.EXECUTOR);
     expr = simplify.simplifyPreservingType(expr);
@@ -353,11 +345,6 @@ public class RexProgramBuilder {
       // Add expression to list, and return a new reference to it.
       ref = addExpr(expr);
       exprMap.put(requireNonNull(key, "key"), ref);
-    } else {
-      if (force) {
-        // Add expression to list, but return the previous ref.
-        addExpr(expr);
-      }
     }
 
     for (;;) {
@@ -670,8 +657,7 @@ public class RexProgramBuilder {
       }
       ref = (RexLocalRef) ref.accept(shuttle);
       this.projectRefList.add(ref);
-      final String name = outFields.get(i).getName();
-      assert name != null;
+      final String name = requireNonNull(outFields.get(i).getName());
       projectNameList.add(name);
     }
     if (conditionRef != null) {
@@ -895,32 +881,37 @@ public class RexProgramBuilder {
   private abstract class RegisterShuttle extends RexShuttle {
     @Override public RexNode visitCall(RexCall call) {
       final RexNode expr = super.visitCall(call);
-      return registerInternal(expr, false);
+      return registerInternal(expr);
     }
 
     @Override public RexNode visitOver(RexOver over) {
       final RexNode expr = super.visitOver(over);
-      return registerInternal(expr, false);
+      return registerInternal(expr);
     }
 
     @Override public RexNode visitLiteral(RexLiteral literal) {
       final RexNode expr = super.visitLiteral(literal);
-      return registerInternal(expr, false);
+      return registerInternal(expr);
     }
 
     @Override public RexNode visitFieldAccess(RexFieldAccess fieldAccess) {
       final RexNode expr = super.visitFieldAccess(fieldAccess);
-      return registerInternal(expr, false);
+      return registerInternal(expr);
     }
 
     @Override public RexNode visitDynamicParam(RexDynamicParam dynamicParam) {
       final RexNode expr = super.visitDynamicParam(dynamicParam);
-      return registerInternal(expr, false);
+      return registerInternal(expr);
     }
 
     @Override public RexNode visitCorrelVariable(RexCorrelVariable variable) {
       final RexNode expr = super.visitCorrelVariable(variable);
-      return registerInternal(expr, false);
+      return registerInternal(expr);
+    }
+
+    @Override public RexNode visitLambda(RexLambda lambda) {
+      super.visitLambda(lambda);
+      return registerInternal(lambda);
     }
   }
 
@@ -991,7 +982,7 @@ public class RexProgramBuilder {
           // Add expression to the list, just so that subsequent
           // expressions don't get screwed up. This expression is
           // unused, so will be eliminated soon.
-          return registerInternal(local, false);
+          return registerInternal(local);
         }
       }
     }
@@ -1057,7 +1048,7 @@ public class RexProgramBuilder {
    * references.
    */
   private static class UpdateRefShuttle extends RexShuttle {
-    private List<RexLocalRef> newRefs;
+    private final List<RexLocalRef> newRefs;
 
     private UpdateRefShuttle(List<RexLocalRef> newRefs) {
       this.newRefs = newRefs;
