@@ -21,6 +21,7 @@ import org.apache.calcite.util.ImmutableNullableList;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 
+import org.checkerframework.checker.nullness.qual.NonNull;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.nullness.qual.PolyNull;
 
@@ -1323,6 +1324,10 @@ public class FlatLists {
       return list.size();
     }
 
+    @Override @NonNull public Object[] toArray(@NonNull ComparableListImpl<T> this) {
+      return this.list.toArray();
+    }
+
     @Override public int compareTo(List o) {
       return compare(list, o);
     }
