@@ -116,16 +116,23 @@ public abstract class SqlCall extends SqlNode {
         getOperandList());
   }
 
+  private boolean needsParentheses(SqlWriter writer, int leftPrec, int rightPrec) {
+    if (getKind() == SqlKind.SET_SEMANTICS_TABLE) {
+      return false;
+    }
+    final SqlOperator operator = getOperator();
+    return leftPrec > operator.getLeftPrec()
+        || (operator.getRightPrec() <= rightPrec && (rightPrec != 0))
+        || writer.isAlwaysUseParentheses() && isA(SqlKind.EXPRESSION)
+        || (operator.getRightPrec() <= rightPrec + 1 && isA(SqlKind.COMPARISON));
+  }
+
   @Override public void unparse(
       SqlWriter writer,
       int leftPrec,
       int rightPrec) {
-    final SqlOperator operator = getOperator();
     final SqlDialect dialect = writer.getDialect();
-    if (leftPrec > operator.getLeftPrec()
-        || (operator.getRightPrec() <= rightPrec && (rightPrec != 0))
-        || writer.isAlwaysUseParentheses() && isA(SqlKind.EXPRESSION)
-        || (operator.getRightPrec() <= rightPrec + 1 && isA(SqlKind.COMPARISON))) {
+    if (needsParentheses(writer, leftPrec, rightPrec)) {
       final SqlWriter.Frame frame = writer.startList("(", ")");
       dialect.unparseCall(writer, this, 0, 0);
       writer.endList(frame);
