@@ -447,6 +447,13 @@ public abstract class ReturnTypes {
   public static final SqlReturnTypeInference CHAR_FORCE_NULLABLE =
       CHAR.andThen(SqlTypeTransforms.FORCE_NULLABLE);
 
+  /**
+   * Type-inference strategy whereby the result type of a call is
+   * CHAR(1), nullable if any argument is nullable.
+   */
+  public static final SqlReturnTypeInference CHAR_NULLABLE_IF_ARGS_NULLABLE =
+      CHAR.andThen(SqlTypeTransforms.TO_NULLABLE);
+
   /**
    * Type-inference strategy whereby the result type of a call is a TINYINT.
    */
@@ -525,6 +532,12 @@ public abstract class ReturnTypes {
   public static final SqlReturnTypeInference VARCHAR =
       ReturnTypes.explicit(SqlTypeName.VARCHAR);
 
+  /**
+   * Type-inference strategy that always returns "VARIANT".
+   */
+  public static final SqlReturnTypeInference VARIANT =
+      ReturnTypes.explicit(SqlTypeName.VARIANT);
+
   /**
    * Type-inference strategy that always returns "VARCHAR" with nulls
    * allowed if any of the operands allow nulls.
@@ -532,6 +545,13 @@ public abstract class ReturnTypes {
   public static final SqlReturnTypeInference VARCHAR_NULLABLE =
       VARCHAR.andThen(SqlTypeTransforms.TO_NULLABLE);
 
+  /**
+   * Type-inference strategy whereby the result type of a call is a nullable
+   * VARCHAR.
+   */
+  public static final SqlReturnTypeInference VARCHAR_FORCE_NULLABLE =
+      VARCHAR.andThen(SqlTypeTransforms.FORCE_NULLABLE);
+
   /**
    * Type-inference strategy that always returns "VARBINARY".
    */
@@ -545,6 +565,13 @@ public abstract class ReturnTypes {
   public static final SqlReturnTypeInference VARBINARY_NULLABLE =
       VARBINARY.andThen(SqlTypeTransforms.TO_NULLABLE);
 
+  /**
+   * Type-inference strategy whereby the result type of a call is a nullable
+   * VARBINARY.
+   */
+  public static final SqlReturnTypeInference VARBINARY_FORCE_NULLABLE =
+      VARBINARY.andThen(SqlTypeTransforms.FORCE_NULLABLE);
+
   /**
    * Type-inference strategy for Histogram agg support.
    */
