@@ -78,6 +78,8 @@ public enum SqlTypeFamily implements RelDataTypeFamily {
   COLUMN_LIST,
   GEO,
   FUNCTION,
+  VARIANT,
+  UUID,
   /** Like ANY, but do not even validate the operand. It may not be an
    * expression. */
   IGNORE;
@@ -119,6 +121,7 @@ public enum SqlTypeFamily implements RelDataTypeFamily {
 
           .put(ExtraSqlTypes.REF_CURSOR, CURSOR)
           .put(Types.ARRAY, ARRAY)
+          .put(Types.JAVA_OBJECT, VARIANT)
           .build();
 
   /**
@@ -222,6 +225,10 @@ public enum SqlTypeFamily implements RelDataTypeFamily {
       return ImmutableList.of(SqlTypeName.COLUMN_LIST);
     case FUNCTION:
       return ImmutableList.of(SqlTypeName.FUNCTION);
+    case UUID:
+      return ImmutableList.of(SqlTypeName.UUID);
+    case VARIANT:
+      return ImmutableList.of(SqlTypeName.VARIANT);
     default:
       throw new IllegalArgumentException();
     }
@@ -281,6 +288,10 @@ public enum SqlTypeFamily implements RelDataTypeFamily {
       return factory.createFunctionSqlType(
           factory.createStructType(ImmutableList.of(), ImmutableList.of()),
           factory.createSqlType(SqlTypeName.ANY));
+    case UUID:
+      return factory.createSqlType(SqlTypeName.UUID);
+    case VARIANT:
+      return factory.createSqlType(SqlTypeName.VARIANT);
     default:
       return null;
     }
