@@ -33,6 +33,7 @@ import org.apache.calcite.sql.SqlFunction;
 import org.apache.calcite.sql.SqlIdentifier;
 import org.apache.calcite.sql.SqlInsert;
 import org.apache.calcite.sql.SqlIntervalQualifier;
+import org.apache.calcite.sql.SqlLambda;
 import org.apache.calcite.sql.SqlLiteral;
 import org.apache.calcite.sql.SqlMatchRecognize;
 import org.apache.calcite.sql.SqlMerge;
@@ -301,6 +302,16 @@ public interface SqlValidator {
    */
   void validateMatchRecognize(SqlCall pattern);
 
+  /**
+   * Validates a lambda expression. lambda expression will be validated twice
+   * during the validation process. The first time is validate lambda expression
+   * namespace, the second time is when validating higher order function operands
+   * type check.
+   *
+   * @param lambdaExpr Lambda expression
+   */
+  void validateLambda(SqlLambda lambdaExpr);
+
   /**
    * Validates a call to an operator.
    *
@@ -466,6 +477,8 @@ public interface SqlValidator {
    */
   SqlValidatorScope getWhereScope(SqlSelect select);
 
+  SqlValidatorScope getMeasureScope(SqlSelect select);
+
   /**
    * Returns the type factory used by this validator.
    *
@@ -602,6 +615,14 @@ public interface SqlValidator {
    */
   SqlValidatorScope getMatchRecognizeScope(SqlMatchRecognize node);
 
+  /**
+   * Returns the lambda expression scope.
+   *
+   * @param node Lambda expression
+   * @return naming scope for lambda expression
+   */
+  SqlValidatorScope getLambdaScope(SqlLambda node);
+
   /**
    * Returns a scope that cannot see anything.
    */
@@ -868,6 +889,27 @@ public interface SqlValidator {
      */
     Config withIdentifierExpansion(boolean expand);
 
+    /**
+     * Returns whether to treat the query being validated as embedded
+     * (as opposed to top-level).
+     *
+     * <p>The default, false, treats the query as top-level;
+     * a value of true treats it as a query inside another, as would be the case
+     * for a view or common table expression.
+     *
+     * <p>Possible behavior differences include ignoring the {@code ORDER BY}
+     * clause of an embedded query, or converting measure expressions of a
+     * non-embedded query into values. */
+    @Value.Default default boolean embeddedQuery() {
+      return false;
+    }
+
+    /**
+     * Sets whether to treat the query being validated as embedded
+     * (as opposed to top-level).
+     */
+    Config withEmbeddedQuery(boolean embedded);
+
     /**
      * Returns whether this validator should be lenient upon encountering an
      * unknown function, default false.
@@ -893,15 +935,39 @@ public interface SqlValidator {
      */
     Config withLenientOperatorLookup(boolean lenient);
 
-    /** Returns whether the validator allows measures to be used without the
-     * AGGREGATE function. Default is true. */
-    @Value.Default default boolean nakedMeasures() {
+    /** Returns whether the validator allows measures to be used without
+     * AGGREGATE function in a non-aggregate query. Default is true.
+     */
+    @Value.Default default boolean nakedMeasuresInNonAggregateQuery() {
       return true;
     }
 
+    /** Sets whether the validator allows measures to be used without AGGREGATE
+     * function in a non-aggregate query.
+     */
+    Config withNakedMeasuresInNonAggregateQuery(boolean value);
+
+    /** Returns whether the validator allows measures to be used without
+     * AGGREGATE function in an aggregate query. Default is true.
+     */
+    @Value.Default default boolean nakedMeasuresInAggregateQuery() {
+      return true;
+    }
+
+    /** Sets whether the validator allows measures to be used without AGGREGATE
+     * function in an aggregate query.
+     */
+    Config withNakedMeasuresInAggregateQuery(boolean value);
+
     /** Sets whether the validator allows measures to be used without the
-     * AGGREGATE function. */
-    Config withNakedMeasures(boolean nakedMeasures);
+     * AGGREGATE function inside or outside aggregate queries.
+     * Deprecated: use the inside / outside variants instead.
+     */
+    @Deprecated // to be removed before 1.38
+    default Config withNakedMeasures(boolean nakedMeasures) {
+      return withNakedMeasuresInAggregateQuery(nakedMeasures)
+              .withNakedMeasuresInNonAggregateQuery(nakedMeasures);
+    }
 
     /** Returns whether the validator supports implicit type coercion. */
     @Value.Default default boolean typeCoercionEnabled() {
