@@ -23,6 +23,7 @@ import org.apache.calcite.rel.RelCollation;
 import org.apache.calcite.rel.RelCollations;
 import org.apache.calcite.rel.RelDistribution;
 import org.apache.calcite.rel.RelFieldCollation;
+import org.apache.calcite.rel.RelHomogeneousShuttle;
 import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.core.Aggregate;
 import org.apache.calcite.rel.core.AggregateCall;
@@ -61,6 +62,7 @@ import org.apache.calcite.rex.RexShuttle;
 import org.apache.calcite.rex.RexSubQuery;
 import org.apache.calcite.rex.RexUtil;
 import org.apache.calcite.rex.RexVisitor;
+import org.apache.calcite.rex.RexVisitorImpl;
 import org.apache.calcite.sql.SqlExplainFormat;
 import org.apache.calcite.sql.SqlExplainLevel;
 import org.apache.calcite.sql.SqlKind;
@@ -78,6 +80,8 @@ import org.apache.calcite.util.mapping.MappingType;
 import org.apache.calcite.util.mapping.Mappings;
 
 import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableSet;
+import com.google.common.collect.Iterables;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
 
@@ -473,6 +477,77 @@ public class RelFieldTrimmer implements ReflectiveVisitor {
     return result(newCalc, mapping, calc);
   }
 
+  /**
+   * Shuttle that finds all {@link TableScan}s inside a given {@link RelNode}.
+   */
+  private static class TableScanCollector extends RelHomogeneousShuttle {
+    private ImmutableSet.Builder<List<String>> builder = ImmutableSet.builder();
+
+    /** Qualified names. */
+    Set<List<String>> tables() {
+      return builder.build();
+    }
+
+    @Override public RelNode visit(TableScan scan) {
+      builder.add(scan.getTable().getQualifiedName());
+      return super.visit(scan);
+    }
+  }
+
+  /**
+   * Shuttle that finds all {@link TableScan}`s inside a given {@link RexNode}.
+   */
+  private static class InputTablesVisitor extends RexVisitorImpl<Void> {
+    private ImmutableSet.Builder<List<String>> builder = ImmutableSet.builder();
+
+    protected InputTablesVisitor() {
+      super(false);
+    }
+
+    /** Qualified names. */
+    Set<List<String>> tables() {
+      return builder.build();
+    }
+
+    @Override public Void visitSubQuery(RexSubQuery subQuery) {
+      if (subQuery.getKind() == SqlKind.SCALAR_QUERY) {
+        subQuery.rel.accept(new RelHomogeneousShuttle() {
+          @Override public RelNode visit(TableScan scan) {
+            builder.add(scan.getTable().getQualifiedName());
+            return super.visit(scan);
+          }
+        });
+      }
+      return null;
+    }
+  }
+
+  private boolean inputContainsSubQueryTables(Project project, RelNode input) {
+    InputTablesVisitor inputSubQueryTablesCollector = new InputTablesVisitor();
+
+    RexUtil.apply(inputSubQueryTablesCollector, project.getProjects(), null);
+
+    Set<List<String>> subQueryTables = inputSubQueryTablesCollector.tables();
+
+    assert subQueryTables.isEmpty() || subQueryTables.size() == 1
+        : "unexpected different tables in subquery: " + subQueryTables;
+
+    TableScanCollector inputTablesCollector = new TableScanCollector();
+    input.accept(inputTablesCollector);
+
+    Set<List<String>> inputTables = inputTablesCollector.tables();
+    // Check for input and subquery tables intersection.
+    if (!subQueryTables.isEmpty()) {
+      for (List<String> t : inputTables) {
+        if (t.equals(Iterables.getOnlyElement(subQueryTables))) {
+          return true;
+        }
+      }
+    }
+
+    return false;
+  }
+
   /**
    * Variant of {@link #trimFields(RelNode, ImmutableBitSet, Set)} for
    * {@link org.apache.calcite.rel.logical.LogicalProject}.
@@ -488,18 +563,24 @@ public class RelFieldTrimmer implements ReflectiveVisitor {
     // Which fields are required from the input?
     final Set<RelDataTypeField> inputExtraFields =
         new LinkedHashSet<>(extraFields);
+
+    // Collect all the SubQueries in the projection list.
+    List<RexSubQuery> subQueries = RexUtil.SubQueryCollector.collect(project);
+    // Get all the correlationIds present in the SubQueries
+    Set<CorrelationId> correlationIds = RelOptUtil.getVariablesUsed(subQueries);
+    // Subquery lookup is required.
+    boolean subQueryLookUp =
+        !correlationIds.isEmpty() && inputContainsSubQueryTables(project, input);
+
     RelOptUtil.InputFinder inputFinder =
-        new RelOptUtil.InputFinder(inputExtraFields);
+        new RelOptUtil.SubQueryAwareInputFinder(inputExtraFields, subQueryLookUp);
+
     for (Ord<RexNode> ord : Ord.zip(project.getProjects())) {
       if (fieldsUsed.get(ord.i)) {
         ord.e.accept(inputFinder);
       }
     }
 
-    // Collect all the SubQueries in the projection list.
-    List<RexSubQuery> subQueries = RexUtil.SubQueryCollector.collect(project);
-    // Get all the correlationIds present in the SubQueries
-    Set<CorrelationId> correlationIds = RelOptUtil.getVariablesUsed(subQueries);
     ImmutableBitSet requiredColumns = ImmutableBitSet.of();
     if (!correlationIds.isEmpty()) {
       assert correlationIds.size() == 1;
@@ -547,6 +628,14 @@ public class RelFieldTrimmer implements ReflectiveVisitor {
       if (fieldsUsed.get(ord.i)) {
         mapping.set(ord.i, newProjects.size());
         RexNode newProjectExpr = ord.e.accept(shuttle);
+        // Subquery need to be remapped
+        if (newProjectExpr instanceof RexSubQuery
+            && newProjectExpr.getKind() == SqlKind.SCALAR_QUERY
+            && !correlationIds.isEmpty()) {
+          newProjectExpr =
+              changeCorrelateReferences((RexSubQuery) newProjectExpr,
+                  Iterables.getOnlyElement(correlationIds), newInput.getRowType(), inputMapping);
+        }
         newProjects.add(newProjectExpr);
       }
     }
@@ -561,6 +650,24 @@ public class RelFieldTrimmer implements ReflectiveVisitor {
     return result(newProject, mapping, project);
   }
 
+  private RexNode changeCorrelateReferences(
+      RexSubQuery node,
+      CorrelationId corrId,
+      RelDataType rowType,
+      Mapping inputMapping) {
+    assert node.getKind() == SqlKind.SCALAR_QUERY : "Expected a SCALAR_QUERY, found "
+        + node.getKind();
+    RelNode subQuery = node.rel;
+    RexBuilder rexBuilder = relBuilder.getRexBuilder();
+
+    RexCorrelVariableMapShuttle rexVisitor =
+        new RexCorrelVariableMapShuttle(corrId, rowType, inputMapping, rexBuilder);
+    RelNode newSubQuery =
+        subQuery.accept(new RexRewritingRelShuttle(rexVisitor));
+
+    return RexSubQuery.scalar(newSubQuery);
+  }
+
   /** Creates a project with a dummy column, to protect the parts of the system
    * that cannot handle a relational expression with no columns.
    *
@@ -1437,9 +1544,12 @@ public class RelFieldTrimmer implements ReflectiveVisitor {
             (RexCorrelVariable) fieldAccess.getReferenceExpr();
         if (referenceExpr.id.equals(correlationId)) {
           int oldIndex = fieldAccess.getField().getIndex();
+          int newIndex = mapping.getTarget(oldIndex);
+          if (newIndex == oldIndex) {
+            return super.visitFieldAccess(fieldAccess);
+          }
           RexNode newCorrel =
               rexBuilder.makeCorrel(newCorrelRowType, referenceExpr.id);
-          int newIndex = mapping.getTarget(oldIndex);
           return rexBuilder.makeFieldAccess(newCorrel, newIndex);
         }
       }
