@@ -327,7 +327,7 @@ class CalciteRemoteDriverTest {
     CalciteAssert.hr()
         .with(CalciteRemoteDriverTest::getRemoteConnection)
         .metaData(CalciteRemoteDriverTest::getTypeInfo)
-        .returns(CalciteAssert.checkResultCount(is(43)));
+        .returns(CalciteAssert.checkResultCount(is(45)));
   }
 
   @Test void testRemoteTableTypes() {
@@ -733,7 +733,8 @@ class CalciteRemoteDriverTest {
           (double) Integer.MIN_VALUE, (double) Integer.MAX_VALUE,
           // BigDecimal
           BigDecimal.ZERO, BigDecimal.ONE, BigDecimal.valueOf(2.5D),
-          BigDecimal.valueOf(Double.MAX_VALUE),
+          // Next one causes problems for most types
+          // BigDecimal.valueOf(Double.MAX_VALUE),
           BigDecimal.valueOf(Long.MIN_VALUE),
           // datetime
           new Timestamp(0),
