@@ -206,8 +206,8 @@ class LatticeTest {
         .doWithConnection(c -> {
           final SchemaPlus schema = c.getRootSchema();
           final SchemaPlus adhoc =
-              requireNonNull(schema.getSubSchema("adhoc"));
-          assertThat(adhoc.getTableNames().contains("EMPLOYEES"), is(true));
+              requireNonNull(schema.subSchemas().get("adhoc"));
+          assertThat(adhoc.tables().get("EMPLOYEES") != null, is(true));
           final CalciteSchema adhocSchema =
               requireNonNull(adhoc.unwrap(CalciteSchema.class));
           final Map.Entry<String, CalciteSchema.LatticeEntry> entry =
@@ -238,8 +238,8 @@ class LatticeTest {
         .doWithConnection(c -> {
           final SchemaPlus schema = c.getRootSchema();
           final SchemaPlus adhoc =
-              requireNonNull(schema.getSubSchema("adhoc"));
-          assertThat(adhoc.getTableNames().contains("EMPLOYEES"), is(true));
+              requireNonNull(schema.subSchemas().get("adhoc"));
+          assertThat(adhoc.tables().get("EMPLOYEES") != null, is(true));
           final CalciteSchema adhocSchema =
               requireNonNull(adhoc.unwrap(CalciteSchema.class));
           final Map.Entry<String, CalciteSchema.LatticeEntry> entry =
@@ -269,8 +269,8 @@ class LatticeTest {
         .doWithConnection(c -> {
           final SchemaPlus schema = c.getRootSchema();
           final SchemaPlus adhoc =
-              requireNonNull(schema.getSubSchema("adhoc"));
-          assertThat(adhoc.getTableNames().contains("EMPLOYEES"), is(true));
+              requireNonNull(schema.subSchemas().get("adhoc"));
+          assertThat(adhoc.tables().get("EMPLOYEES") != null, is(true));
           final CalciteSchema adhocSchema =
               requireNonNull(adhoc.unwrap(CalciteSchema.class));
           final Map.Entry<String, CalciteSchema.LatticeEntry> entry =
