@@ -56,6 +56,13 @@ public interface ElasticsearchRel extends RelNode {
      */
     final List<Map.Entry<String, RelFieldCollation.Direction>> sort = new ArrayList<>();
 
+    /**
+     * Sorting missing values.
+     *
+     * @see <a href="https://www.elastic.co/guide/en/elasticsearch/reference/current/sort-search-results.html#_missing_values">Missing Values</a>
+     */
+    final List<Map.Entry<String, RelFieldCollation.NullDirection>> nullsSort = new ArrayList<>();
+
     /**
      * Elastic aggregation ({@code MIN / MAX / COUNT} etc.) statements (functions).
      *
@@ -110,6 +117,11 @@ public interface ElasticsearchRel extends RelNode {
       sort.add(Pair.of(field, direction));
     }
 
+    void addNullsSort(String field, RelFieldCollation.NullDirection nullDirection) {
+      requireNonNull(field, "field");
+      nullsSort.add(new Pair<>(field, nullDirection));
+    }
+
     void addAggregation(String field, String expression) {
       requireNonNull(field, "field");
       requireNonNull(expression, "expression");
