@@ -21,7 +21,8 @@ import com.fasterxml.jackson.core.JsonGenerator;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Objects;
+
+import static java.util.Objects.requireNonNull;
 
 /**
  * Utility class to generate elastic search queries. Most query builders have
@@ -236,25 +237,25 @@ class QueryBuilders {
     private final List<QueryBuilder> shouldClauses = new ArrayList<>();
 
     BoolQueryBuilder must(QueryBuilder queryBuilder) {
-      Objects.requireNonNull(queryBuilder, "queryBuilder");
+      requireNonNull(queryBuilder, "queryBuilder");
       mustClauses.add(queryBuilder);
       return this;
     }
 
     BoolQueryBuilder filter(QueryBuilder queryBuilder) {
-      Objects.requireNonNull(queryBuilder, "queryBuilder");
+      requireNonNull(queryBuilder, "queryBuilder");
       filterClauses.add(queryBuilder);
       return this;
     }
 
     BoolQueryBuilder mustNot(QueryBuilder queryBuilder) {
-      Objects.requireNonNull(queryBuilder, "queryBuilder");
+      requireNonNull(queryBuilder, "queryBuilder");
       mustNotClauses.add(queryBuilder);
       return this;
     }
 
     BoolQueryBuilder should(QueryBuilder queryBuilder) {
-      Objects.requireNonNull(queryBuilder, "queryBuilder");
+      requireNonNull(queryBuilder, "queryBuilder");
       shouldClauses.add(queryBuilder);
       return this;
     }
@@ -298,8 +299,8 @@ class QueryBuilders {
     private final Object value;
 
     private TermQueryBuilder(final String fieldName, final Object value) {
-      this.fieldName = Objects.requireNonNull(fieldName, "fieldName");
-      this.value = Objects.requireNonNull(value, "value");
+      this.fieldName = requireNonNull(fieldName, "fieldName");
+      this.value = requireNonNull(value, "value");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
@@ -321,8 +322,8 @@ class QueryBuilders {
     private final Iterable<?> values;
 
     private TermsQueryBuilder(final String fieldName, final Iterable<?> values) {
-      this.fieldName = Objects.requireNonNull(fieldName, "fieldName");
-      this.values = Objects.requireNonNull(values, "values");
+      this.fieldName = requireNonNull(fieldName, "fieldName");
+      this.values = requireNonNull(values, "values");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
@@ -350,8 +351,8 @@ class QueryBuilders {
     private final Object value;
 
     private MatchQueryBuilder(final String fieldName, final Object value) {
-      this.fieldName = Objects.requireNonNull(fieldName, "fieldName");
-      this.value = Objects.requireNonNull(value, "value");
+      this.fieldName = requireNonNull(fieldName, "fieldName");
+      this.value = requireNonNull(value, "value");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
@@ -374,8 +375,8 @@ class QueryBuilders {
     private final Iterable<?> values;
 
     private MatchesQueryBuilder(final String fieldName, final Iterable<?> values) {
-      this.fieldName = Objects.requireNonNull(fieldName, "fieldName");
-      this.values = Objects.requireNonNull(values, "values");
+      this.fieldName = requireNonNull(fieldName, "fieldName");
+      this.values = requireNonNull(values, "values");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
@@ -419,17 +420,17 @@ class QueryBuilders {
     private String format;
 
     private RangeQueryBuilder(final String fieldName) {
-      this.fieldName = Objects.requireNonNull(fieldName, "fieldName");
+      this.fieldName = requireNonNull(fieldName, "fieldName");
     }
 
     private RangeQueryBuilder to(Object value, boolean lte) {
-      this.lt = Objects.requireNonNull(value, "value");
+      this.lt = requireNonNull(value, "value");
       this.lte = lte;
       return this;
     }
 
     private RangeQueryBuilder from(Object value, boolean gte) {
-      this.gt = Objects.requireNonNull(value, "value");
+      this.gt = requireNonNull(value, "value");
       this.gte = gte;
       return this;
     }
@@ -502,7 +503,7 @@ class QueryBuilders {
       this.value = value;
     }
 
-    @Override void writeJson(final JsonGenerator generator) throws IOException {
+    @Override void writeJson(final JsonGenerator generator) {
       throw new UnsupportedOperationException();
     }
   }
@@ -514,7 +515,7 @@ class QueryBuilders {
     private final String fieldName;
 
     ExistsQueryBuilder(final String fieldName) {
-      this.fieldName = Objects.requireNonNull(fieldName, "fieldName");
+      this.fieldName = requireNonNull(fieldName, "fieldName");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
@@ -536,7 +537,7 @@ class QueryBuilders {
     private final QueryBuilder builder;
 
     private ConstantScoreQueryBuilder(final QueryBuilder builder) {
-      this.builder = Objects.requireNonNull(builder, "builder");
+      this.builder = requireNonNull(builder, "builder");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
@@ -559,7 +560,7 @@ class QueryBuilders {
     private final QueryBuilder builder;
 
     private DisMaxQueryBuilder(final QueryBuilder builder) {
-      this.builder = Objects.requireNonNull(builder, "builder");
+      this.builder = requireNonNull(builder, "builder");
     }
 
     @Override void writeJson(final JsonGenerator generator) throws IOException {
