@@ -27,7 +27,8 @@ import com.google.common.collect.ImmutableList;
 
 import java.math.BigDecimal;
 import java.util.List;
-import java.util.Objects;
+
+import static java.util.Objects.requireNonNull;
 
 /** Abstract syntax tree.
  *
@@ -136,8 +137,8 @@ public class Ast {
     public final SqlParserPos pos;
 
     protected Node(SqlParserPos pos, Op op) {
-      this.op = Objects.requireNonNull(op, "op");
-      this.pos = Objects.requireNonNull(pos, "pos");
+      this.op = requireNonNull(op, "op");
+      this.pos = requireNonNull(pos, "pos");
     }
   }
 
@@ -154,7 +155,7 @@ public class Ast {
 
     protected Assignment(SqlParserPos pos, Op op, Identifier target) {
       super(pos, op);
-      this.target = Objects.requireNonNull(target, "target");
+      this.target = requireNonNull(target, "target");
     }
   }
 
@@ -164,7 +165,7 @@ public class Ast {
 
     public LoadStmt(SqlParserPos pos, Identifier target, Literal name) {
       super(pos, Op.LOAD, target);
-      this.name = Objects.requireNonNull(name, "name");
+      this.name = requireNonNull(name, "name");
     }
   }
 
@@ -333,7 +334,7 @@ public class Ast {
 
     public DumpStmt(SqlParserPos pos, Identifier relation) {
       super(pos, Op.DUMP);
-      this.relation = Objects.requireNonNull(relation, "relation");
+      this.relation = requireNonNull(relation, "relation");
     }
   }
 
@@ -343,7 +344,7 @@ public class Ast {
 
     public DescribeStmt(SqlParserPos pos, Identifier relation) {
       super(pos, Op.DESCRIBE);
-      this.relation = Objects.requireNonNull(relation, "relation");
+      this.relation = requireNonNull(relation, "relation");
     }
   }
 
@@ -353,7 +354,7 @@ public class Ast {
 
     public Literal(SqlParserPos pos, Object value) {
       super(pos, Op.LITERAL);
-      this.value = Objects.requireNonNull(value, "value");
+      this.value = requireNonNull(value, "value");
     }
 
     public static NumericLiteral createExactNumeric(String s,
@@ -408,7 +409,7 @@ public class Ast {
 
     public Identifier(SqlParserPos pos, String value) {
       super(pos, Op.IDENTIFIER);
-      this.value = Objects.requireNonNull(value, "value");
+      this.value = requireNonNull(value, "value");
     }
 
     public boolean isStar() {
@@ -466,8 +467,8 @@ public class Ast {
 
     public FieldSchema(SqlParserPos pos, Identifier id, Type type) {
       super(pos, Op.FIELD_SCHEMA);
-      this.id = Objects.requireNonNull(id, "id");
-      this.type = Objects.requireNonNull(type, "type");
+      this.id = requireNonNull(id, "id");
+      this.type = requireNonNull(type, "type");
     }
   }
 
