@@ -25,6 +25,7 @@ import org.apache.calcite.rex.RexNode;
 import org.apache.calcite.runtime.PairList;
 import org.apache.calcite.runtime.Utilities;
 import org.apache.calcite.sql.SqlCall;
+import org.apache.calcite.sql.SqlKind;
 import org.apache.calcite.sql.SqlLiteral;
 import org.apache.calcite.sql.SqlNode;
 import org.apache.calcite.sql.SqlOperator;
@@ -63,7 +64,6 @@ import static org.apache.calcite.test.Matchers.relIsValid;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.hasSize;
-import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 
 import static java.util.Objects.requireNonNull;
@@ -185,7 +185,7 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
       final RelDataType rowType =
           validator.getValidatedNodeType(validatedNode);
       final List<RelDataTypeField> fields = rowType.getFieldList();
-      assertEquals(1, fields.size(), "expected query to return 1 field");
+      assertThat("expected query to return 1 field", fields, hasSize(1));
       final RelDataType actualType = fields.get(0).getType();
       String actual = SqlTests.getTypeString(actualType);
       assertThat(actual, matcher);
@@ -238,8 +238,8 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
     typeChecker.checkType(() -> "Query: " + query, actualType);
 
     Pair<SqlValidator, SqlNode> p = parseAndValidate(factory, query);
-    SqlValidator validator = requireNonNull(p.left);
-    SqlNode n = requireNonNull(p.right);
+    SqlValidator validator = p.left;
+    SqlNode n = p.right;
     final RelDataType parameterRowType = validator.getParameterRowType(n);
     parameterChecker.checkParameters(parameterRowType);
   }
@@ -247,16 +247,16 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
   @Override public void validateAndThen(SqlTestFactory factory,
       StringAndPos sap, ValidatedNodeConsumer consumer) {
     Pair<SqlValidator, SqlNode> p = parseAndValidate(factory, sap.sql);
-    SqlValidator validator = requireNonNull(p.left);
-    SqlNode rewrittenNode = requireNonNull(p.right);
+    SqlValidator validator = p.left;
+    SqlNode rewrittenNode = p.right;
     consumer.accept(sap, validator, rewrittenNode);
   }
 
   @Override public <R> R validateAndApply(SqlTestFactory factory,
       StringAndPos sap, ValidatedNodeFunction<R> function) {
     Pair<SqlValidator, SqlNode> p = parseAndValidate(factory, sap.sql);
-    SqlValidator validator = requireNonNull(p.left);
-    SqlNode rewrittenNode = requireNonNull(p.right);
+    SqlValidator validator = p.left;
+    SqlNode rewrittenNode = p.right;
     return function.apply(sap, validator, rewrittenNode);
   }
 
@@ -326,7 +326,7 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
    * @return Query that evaluates a scalar expression
    */
   protected String buildQuery2(SqlTestFactory factory, String expression) {
-    if (expression.matches("(?i).*(percentile_(cont|disc)|convert|sort_array)\\(.*")) {
+    if (expression.matches("(?i).*(percentile_(cont|disc)|convert|sort_array|cast)\\(.*")) {
       // PERCENTILE_CONT requires its argument to be a literal,
       // so converting its argument to a column will cause false errors.
       // Similarly, MSSQL-style CONVERT.
@@ -361,15 +361,18 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
             return literal;
           }
 
-          @Override public SqlNode visit(SqlCall call) {
+          @Override public @Nullable SqlNode visit(SqlCall call) {
             SqlOperator operator = call.getOperator();
+            if (operator.getKind() == SqlKind.LAMBDA) {
+              return call;
+            }
             if (operator instanceof SqlUnresolvedFunction) {
               final SqlUnresolvedFunction unresolvedFunction =
                   (SqlUnresolvedFunction) operator;
               final SqlOperator lookup =
                   SqlValidatorUtil.lookupSqlFunctionByID(
                       SqlStdOperatorTable.instance(),
-                      unresolvedFunction.getSqlIdentifier(),
+                      requireNonNull(unresolvedFunction.getSqlIdentifier()),
                       unresolvedFunction.getFunctionType());
               if (lookup != null) {
                 operator = lookup;
@@ -382,6 +385,11 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
                 && isNull(call.operand(0))) {
               literalSet.add(call);
               return call;
+            } else if (operator == SqlStdOperatorTable.TRIM) {
+              // see https://issues.apache.org/jira/projects/CALCITE/issues/CALCITE-6780
+              // don't extract trimmed literal for TRIM function
+              call.operand(2).accept(this);
+              return call;
             } else if (ops.contains(operator)) {
               // "Argument to function 'LOCALTIME' must be a
               // literal"
@@ -480,8 +488,8 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
     String sql2 = diffRepos.expand("sql", sql);
     final Pair<SqlValidator, RelRoot> pair =
         convertSqlToRel2(factory, sql2, decorrelate, trim);
-    final RelRoot root = requireNonNull(pair.right);
-    final SqlValidator validator = requireNonNull(pair.left);
+    final RelRoot root = pair.right;
+    final SqlValidator validator = pair.left;
     RelNode rel = root.project();
 
     assertNotNull(rel);
